@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2009,2010,2012 Red Hat, Inc.
+# Copyright (C) 2009-2014 Red Hat, Inc.
 #
 # Authors:
 # Thomas Woerner <twoerner@redhat.com>
@@ -30,6 +30,7 @@ import slip.dbus
 
 from firewall.config import *
 from firewall.config.dbus import *
+from firewall.core.base import DEFAULT_ZONE_TARGET
 from firewall.dbus_utils import dbus_to_python
 import dbus
 from decorator import decorator
@@ -37,32 +38,39 @@ from firewall.functions import b2u
 from firewall.core.rich import Rich_Rule
 
 exception_handler = None
+not_authorized_loop = False
 
 @decorator
 def handle_exceptions(func, *args, **kwargs):
     """Decorator to handle exceptions
     """
     global exception_handler
-    #while 1:
-    try:
-        return func(*args, **kwargs)
-    except dbus.exceptions.DBusException as e:
-        dbus_message = e.get_dbus_message() # returns unicode
-        dbus_name = e.get_dbus_name()
-        if not exception_handler:
-            raise
-        if "NotAuthorizedException" in dbus_name:
-            exception_handler("NotAuthorizedException")
-        else:
-            if dbus_message:
-                exception_handler(dbus_message)
+    global not_authorized_loop
+    authorized = False
+    while not authorized:
+        try:
+            return func(*args, **kwargs)
+        except dbus.exceptions.DBusException as e:
+            dbus_message = e.get_dbus_message() # returns unicode
+            dbus_name = e.get_dbus_name()
+            if not exception_handler:
+                raise
+            if "NotAuthorizedException" in dbus_name:
+                authorized = False
+                exception_handler("NotAuthorizedException")
+            else:
+                authorized = True
+                if dbus_message:
+                    exception_handler(dbus_message)
+                else:
+                    exception_handler(b2u(str(e)))
+        except Exception as e:
+            if not exception_handler:
+                raise
             else:
                 exception_handler(b2u(str(e)))
-    except Exception as e:
-        if not exception_handler:
-            raise
-        else:
-            exception_handler(b2u(str(e)))
+        if not not_authorized_loop:
+            break
 
 # zone config setings
 
@@ -72,8 +80,12 @@ class FirewallClientZoneSettings(object):
         if settings:
             self.settings = settings
         else:
-            self.settings = ["", "", "", False, "", [], [], [], False, [],
-                             [], [], []]
+            self.settings = ["", "", "", False, DEFAULT_ZONE_TARGET, [], [],
+                             [], False, [], [], [], []]
+
+    @handle_exceptions
+    def __repr__(self):
+        return '%s(%r)' % (self.__class__, self.settings)
 
     @handle_exceptions
     def getVersion(self):
@@ -100,10 +112,10 @@ class FirewallClientZoneSettings(object):
 
     @handle_exceptions
     def getTarget(self):
-        return self.settings[4]
+        return self.settings[4] if self.settings[4] != DEFAULT_ZONE_TARGET else "default"
     @handle_exceptions
     def setTarget(self, target):
-        self.settings[4] = target
+        self.settings[4] = target if target != "default" else DEFAULT_ZONE_TARGET
 
     @handle_exceptions
     def getServices(self):
@@ -321,6 +333,10 @@ class FirewallClientServiceSettings(object):
         else:
             self.settings = ["", "", "", [], [], {}]
 
+    @handle_exceptions
+    def __repr__(self):
+        return '%s(%r)' % (self.__class__, self.settings)
+
     @handle_exceptions
     def getVersion(self):
         return self.settings[0]
@@ -461,6 +477,10 @@ class FirewallClientIcmpTypeSettings(object):
         else:
             self.settings = ["", "", "", []]
 
+    @handle_exceptions
+    def __repr__(self):
+        return '%s(%r)' % (self.__class__, self.settings)
+
     @handle_exceptions
     def getVersion(self):
         return self.settings[0]
@@ -572,6 +592,10 @@ class FirewallClientPoliciesLockdownWhitelist(object):
         else:
             self.settings = [ [], [], [], [] ]
 
+    @handle_exceptions
+    def __repr__(self):
+        return '%s(%r)' % (self.__class__, self.settings)
+
     @handle_exceptions
     def getCommands(self):
         return sorted(self.settings[0])
@@ -676,6 +700,10 @@ class FirewallClientDirect(object):
         else:
             self.settings = [ [], [], [], ]
 
+    @handle_exceptions
+    def __repr__(self):
+        return '%s(%r)' % (self.__class__, self.settings)
+
     @handle_exceptions
     def getAllChains(self):
         return self.settings[0]
@@ -723,6 +751,11 @@ class FirewallClientDirect(object):
         if idx in self.settings[1]:
             self.settings[1].remove(idx)
     @handle_exceptions
+    def removeRules(self, ipv, table, chain):
+        for idx in list(self.settings[1]):
+            if idx[0] == ipv and idx[1] == table and idx[2] == chain:
+                self.settings[1].remove(idx)
+    @handle_exceptions
     def queryRule(self, ipv, table, chain, priority, args):
         idx = (ipv, table, chain, priority, args)
         return idx in self.settings[1]
@@ -1030,6 +1063,16 @@ class FirewallClient(object):
         global exception_handler
         exception_handler = handler
 
+    @handle_exceptions
+    def getNotAuthorizedLoop(self):
+        global not_authorized_loop
+        return not_authorized_loop
+
+    @handle_exceptions
+    def setNotAuthorizedLoop(self, enable):
+        global not_authorized_loop
+        not_authorized_loop = (enable == True)
+
     @handle_exceptions
     def connect(self, name, callback, *args):
         if name in self._callbacks:
@@ -1184,6 +1227,12 @@ class FirewallClient(object):
 
     # list functions
 
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def getZoneSettings(self, zone):
+        return FirewallClientZoneSettings(list(dbus_to_python(\
+                    self.fw.getZoneSettings(zone))))
+
     @slip.dbus.polkit.enable_proxy
     @handle_exceptions
     def listServices(self):
@@ -1490,6 +1539,11 @@ class FirewallClient(object):
     def removeRule(self, ipv, table, chain, priority, args):
         self.fw_direct.removeRule(ipv, table, chain, priority, args)
 
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def removeRules(self, ipv, table, chain):
+        self.fw_direct.removeRules(ipv, table, chain)
+
     @slip.dbus.polkit.enable_proxy
     @handle_exceptions
     def queryRule(self, ipv, table, chain, priority, args):
@@ -1619,3 +1673,8 @@ class FirewallClient(object):
     def removeLockdownWhitelistUser(self, user):
         self.fw_policies.removeLockdownWhitelistUser(user)
 
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def authorizeAll(self):
+        """ Authorize once for all polkit actions. """
+        self.fw.authorizeAll()
