@@ -38,7 +38,7 @@ APPLET_NAME = 'firewall-applet'
 DATADIR = '/usr/share/' + DAEMON_NAME
 CONFIG_GLADE_NAME = CONFIG_NAME + '.glade'
 COPYRIGHT = '(C) 2010-2012 Red Hat, Inc.'
-VERSION = '0.3.10'
+VERSION = '0.3.11'
 AUTHORS = [
     "Thomas Woerner <twoerner@redhat.com>",
     "Jiri Popelka <jpopelka@redhat.com>",
@@ -80,3 +80,6 @@ SYSCTL_CONFIG = '/etc/sysctl.conf'
 # fallbacks: will be overloaded by firewalld.conf
 FALLBACK_ZONE = "public"
 FALLBACK_MINIMAL_MARK = 100
+FALLBACK_CLEANUP_ON_EXIT = "yes"
+FALLBACK_LOCKDOWN = "no"
+FALLBACK_IPV6_RPFILTER = "yes"
