@@ -82,19 +82,49 @@ class FirewallDirect:
                     except FirewallError as error:
                         log.warning(str(error))
 
+    def _check_ipv_table(self, ipv, table):
+        ipvs = [ 'ipv4', 'ipv6', 'eb' ]
+        if ipv not in ipvs:
+            raise FirewallError(INVALID_IPV,
+                                "'%s' not in '%s'" % (ipv, ipvs))
+        tables = ipXtables.BUILT_IN_CHAINS.keys() if ipv in [ 'ipv4', 'ipv6' ] \
+                                         else ebtables.BUILT_IN_CHAINS.keys()
+        if table not in tables:
+            raise FirewallError(INVALID_TABLE,
+                                "'%s' not in '%s'" % (table, tables))
+
+    def _check_builtin_chain(self, ipv, table, chain):
+        if ipv in ['ipv4', 'ipv6']:
+            built_in_chains = ipXtables.BUILT_IN_CHAINS[table]
+            our_chains = ipXtables.OUR_CHAINS[table]
+        else:
+            built_in_chains = ebtables.BUILT_IN_CHAINS[table]
+            our_chains = ebtables.OUR_CHAINS[table]
+        if chain in built_in_chains:
+            raise FirewallError(BUILTIN_CHAIN,
+                 "chain '%s' is built-in chain" % chain)
+        if chain in our_chains:
+            raise FirewallError(BUILTIN_CHAIN,
+                 "chain '%s' is reserved" % chain)
+
+
     # DIRECT CHAIN
 
     def __chain(self, add, ipv, table, chain):
+        self._check_ipv_table(ipv, table)
+        self._check_builtin_chain(ipv, table, chain)
         table_id = (ipv, table)
 
         if add:
             if table_id in self._chains and \
                     chain in self._chains[table_id]:
-                raise FirewallError(ALREADY_ENABLED)
+                raise FirewallError(ALREADY_ENABLED,
+                     "chain '%s' already is in '%s:%s'" % (chain, ipv, table))
         else:
             if table_id not in self._chains or \
                     not chain in self._chains[table_id]:
-                raise FirewallError(NOT_ENABLED)
+                raise FirewallError(NOT_ENABLED,
+                    "chain '%s' is not in '%s:%s'" % (chain, ipv, table))
 
         rule = [ "-t", table ]
         if add:
@@ -124,11 +154,14 @@ class FirewallDirect:
         self.__chain(False, ipv, table, chain)
 
     def query_chain(self, ipv, table, chain):
+        self._check_ipv_table(ipv, table)
+        self._check_builtin_chain(ipv, table, chain)
         table_id = (ipv, table)
-        return (table_id in self._chains and \
-                    chain in self._chains[table_id])
+        return (table_id in self._chains and
+                   chain in self._chains[table_id])
 
     def get_chains(self, ipv, table):
+        self._check_ipv_table(ipv, table)
         table_id = (ipv, table)
         if table_id in self._chains:
             return self._chains[table_id]
@@ -145,13 +178,14 @@ class FirewallDirect:
     # DIRECT RULE
 
     def __rule(self, enable, ipv, table, chain, priority, args):
+        self._check_ipv_table(ipv, table)
         _chain = chain
         # use "%s_chain" for built-in chains
 
         if ipv in [ "ipv4", "ipv6" ]:
-            _CHAINS = ipXtables.CHAINS
+            _CHAINS = ipXtables.BUILT_IN_CHAINS
         else:
-            _CHAINS = ebtables.CHAINS
+            _CHAINS = ebtables.BUILT_IN_CHAINS
 
         if table in _CHAINS and chain in _CHAINS[table]:
             _chain = "%s_direct" % (chain)
@@ -162,11 +196,16 @@ class FirewallDirect:
         if enable:
             if chain_id in self._rules and \
                     rule_id in self._rules[chain_id]:
-                raise FirewallError(ALREADY_ENABLED)
+                raise FirewallError(ALREADY_ENABLED,
+                                    "rule '%s' already is in '%s:%s:%s'" % \
+                                    (args, ipv, table, chain))
         else:
             if not chain_id in self._rules or \
                     not rule_id in self._rules[chain_id]:
-                raise FirewallError(NOT_ENABLED)
+                raise FirewallError(NOT_ENABLED,
+                                    "rule '%s' is not in '%s:%s:%s'" % \
+                                    (args, ipv, table, chain))
+
             # get priority of rule
             priority = self._rules[chain_id][rule_id]
 
@@ -253,11 +292,13 @@ class FirewallDirect:
         self.__rule(False, ipv, table, chain, priority, args)
 
     def query_rule(self, ipv, table, chain, priority, args):
+        self._check_ipv_table(ipv, table)
         chain_id = (ipv, table, chain)
         return (chain_id in self._rules and \
                 (priority, args) in self._rules[chain_id])
 
     def get_rules(self, ipv, table, chain):
+        self._check_ipv_table(ipv, table)
         chain_id = (ipv, table, chain)
         if chain_id in self._rules:
             return list(self._rules[chain_id].keys())
