@@ -26,7 +26,8 @@ from firewall.functions import portStr, checkIPnMask, checkIP6nMask, \
     checkProtocol, enable_ip_forwarding, check_single_address
 from firewall.core.rich import *
 from firewall.errors import *
-from firewall.core.ipXtables import ip4tables_available_tables, ip6tables_available_tables
+from firewall.core.ipXtables import ip4tables_available_tables,\
+    ip6tables_available_tables, OUR_CHAINS
 
 mangle = []
 if "mangle" in ip4tables_available_tables:
@@ -187,6 +188,10 @@ class FirewallZone:
                 ipvs.append("ipv6")
 
             for ipv in ipvs:
+                OUR_CHAINS[table].update(set([_zone,
+                                              "%s_log" % _zone,
+                                              "%s_deny" % _zone,
+                                              "%s_allow" % _zone]))
                 chains.append((ipv, [ _zone, "-t", table ]))
                 chains.append((ipv, [ "%s_log" % (_zone), "-t", table ]))
                 chains.append((ipv, [ "%s_deny" % (_zone), "-t", table ]))
@@ -242,6 +247,7 @@ class FirewallZone:
             ret = self._fw.handle_chains(chains, create)
             if ret:
                 # also create rules
+                (cleanup_rules, msg) = ret
                 self._fw.handle_rules(cleanup_rules, not create)
 
                 (cleanup_chains, msg) = ret
@@ -473,9 +479,11 @@ class FirewallZone:
         interface_id = self.__interface_id(interface)
 
         if interface_id in _obj.settings["interfaces"]:
-            raise FirewallError(ZONE_ALREADY_SET)
+            raise FirewallError(ZONE_ALREADY_SET,
+                         "'%s' already bound to '%s'" % (interface_id, zone))
         if self.get_zone_of_interface(interface) != None:
-            raise FirewallError(ZONE_CONFLICT)
+            raise FirewallError(ZONE_CONFLICT,
+                                "'%s' already bound to a zone" % interface)
 
         self.__interface(True, _zone, interface)
 
@@ -511,10 +519,13 @@ class FirewallZone:
         self._fw.check_panic()
         zoi = self.get_zone_of_interface(interface)
         if zoi == None:
-            raise FirewallError(UNKNOWN_INTERFACE, interface)
+            raise FirewallError(UNKNOWN_INTERFACE,
+                                "'%s' is not in any zone" % interface)
         _zone = zoi if zone == "" else self._fw.check_zone(zone)
         if zoi != _zone:
-            raise FirewallError(ZONE_CONFLICT)
+            raise FirewallError(ZONE_CONFLICT,
+                                "remove_interface(%s, %s): zoi='%s'" % \
+                                (zone, interface, zoi))
 
         _obj = self._zones[_zone]
         interface_id = self.__interface_id(interface)
@@ -609,9 +620,11 @@ class FirewallZone:
         source_id = self.__source_id(source)
 
         if source_id in _obj.settings["sources"]:
-            raise FirewallError(ZONE_ALREADY_SET)
+            raise FirewallError(ZONE_ALREADY_SET,
+                            "'%s' already bound to '%s'" % (source_id, zone))
         if self.get_zone_of_source(source) != None:
-            raise FirewallError(ZONE_CONFLICT)
+            raise FirewallError(ZONE_CONFLICT,
+                                "'%s' already bound to a zone" % source_id)
 
         self.__source(True, _zone, source_id[0], source_id[1])
 
@@ -639,10 +652,13 @@ class FirewallZone:
         self._fw.check_panic()
         zos = self.get_zone_of_source(source)
         if zos == None:
-            raise FirewallError(UNKNOWN_SOURCE, source)
+            raise FirewallError(UNKNOWN_SOURCE,
+                                "'%s' is not in any zone" % source)
         _zone = zos if zone == "" else self._fw.check_zone(zone)
         if zos != _zone:
-            raise FirewallError(ZONE_CONFLICT)
+            raise FirewallError(ZONE_CONFLICT,
+                                "remove_source(%s, %s): zos='%s'" % \
+                                (zone, source, zos))
 
         _obj = self._zones[_zone]
         source_id = self.__source_id(source)
@@ -1004,12 +1020,14 @@ class FirewallZone:
 
     def add_rule(self, zone, rule, timeout=0, sender=None):
         _zone = self._fw.check_zone(zone)
+        self._fw.check_timeout(timeout)
         self._fw.check_panic()
         _obj = self._zones[_zone]
 
         rule_id = self.__rule_id(rule)
         if rule_id in _obj.settings["rules"]:
-            raise FirewallError(ALREADY_ENABLED)
+            raise FirewallError(ALREADY_ENABLED,
+                                "'%s' already in '%s'" % (rule, zone))
 
         if _obj.applied:
             mark = self.__rule(True, _zone, rule, None)
@@ -1028,7 +1046,8 @@ class FirewallZone:
 
         rule_id = self.__rule_id(rule)
         if not rule_id in _obj.settings["rules"]:
-            raise FirewallError(NOT_ENABLED)
+            raise FirewallError(NOT_ENABLED,
+                                "'%s' not in '%s'" % (rule, zone))
 
         if "mark" in _obj.settings["rules"][rule_id]:
             mark = _obj.settings["rules"][rule_id]["mark"]
@@ -1115,12 +1134,14 @@ class FirewallZone:
 
     def add_service(self, zone, service, timeout=0, sender=None):
         _zone = self._fw.check_zone(zone)
+        self._fw.check_timeout(timeout)
         self._fw.check_panic()
         _obj = self._zones[_zone]
 
         service_id = self.__service_id(service)
         if service_id in _obj.settings["services"]:
-            raise FirewallError(ALREADY_ENABLED)
+            raise FirewallError(ALREADY_ENABLED,
+                                "'%s' already in '%s'" % (service_id, zone))
 
         if _obj.applied:
             self.__service(True, _zone, service)
@@ -1137,7 +1158,8 @@ class FirewallZone:
 
         service_id = self.__service_id(service)
         if not service_id in _obj.settings["services"]:
-            raise FirewallError(NOT_ENABLED)
+            raise FirewallError(NOT_ENABLED,
+                                "'%s' not in '%s'" % (service, zone))
 
         if _obj.applied:
             self.__service(False, _zone, service)
@@ -1190,12 +1212,14 @@ class FirewallZone:
 
     def add_port(self, zone, port, protocol, timeout=0, sender=None):
         _zone = self._fw.check_zone(zone)
+        self._fw.check_timeout(timeout)
         self._fw.check_panic()
         _obj = self._zones[_zone]
 
         port_id = self.__port_id(port, protocol)
         if port_id in _obj.settings["ports"]:
-            raise FirewallError(ALREADY_ENABLED)
+            raise FirewallError(ALREADY_ENABLED,
+                          "'%s:%s' already in '%s'" % (port, protocol, zone))
 
         if _obj.applied:
             self.__port(True, _zone, port, protocol)
@@ -1212,7 +1236,8 @@ class FirewallZone:
 
         port_id = self.__port_id(port, protocol)
         if not port_id in _obj.settings["ports"]:
-            raise FirewallError(NOT_ENABLED)
+            raise FirewallError(NOT_ENABLED,
+                             "'%s:%s' not in '%s'" % (port, protocol, zone))
 
         if _obj.applied:
             self.__port(False, _zone, port, protocol)
@@ -1270,12 +1295,14 @@ class FirewallZone:
 
     def add_masquerade(self, zone, timeout=0, sender=None):
         _zone = self._fw.check_zone(zone)
+        self._fw.check_timeout(timeout)
         self._fw.check_panic()
         _obj = self._zones[_zone]
 
         masquerade_id = self.__masquerade_id()
         if masquerade_id in _obj.settings["masquerade"]:
-            raise FirewallError(ALREADY_ENABLED)
+            raise FirewallError(ALREADY_ENABLED,
+                                "masquerade already enabled in '%s'" % zone)
 
         if _obj.applied:
             self.__masquerade(True, _zone)
@@ -1292,7 +1319,8 @@ class FirewallZone:
 
         masquerade_id = self.__masquerade_id()
         if masquerade_id not in _obj.settings["masquerade"]:
-            raise FirewallError(NOT_ENABLED)
+            raise FirewallError(NOT_ENABLED,
+                                "masquerade not enabled in '%s'" % zone)
 
         if _obj.applied:
             self.__masquerade(False, _zone)
@@ -1316,7 +1344,8 @@ class FirewallZone:
             if not check_single_address(ipv, toaddr):
                 raise FirewallError(INVALID_ADDR, toaddr)
         if not toport and not toaddr:
-            raise FirewallError(INVALID_FORWARD)
+            raise FirewallError(INVALID_FORWARD,
+                            "port-forwarding is missing to-port AND to-addr")
 
     def __forward_port_id(self, port, protocol, toport=None, toaddr=None):
         self.check_forward_port("ipv4", port, protocol, toport, toaddr)
@@ -1386,12 +1415,15 @@ class FirewallZone:
     def add_forward_port(self, zone, port, protocol, toport=None,
                          toaddr=None, timeout=0, sender=None):
         _zone = self._fw.check_zone(zone)
+        self._fw.check_timeout(timeout)
         self._fw.check_panic()
         _obj = self._zones[_zone]
 
         forward_id = self.__forward_port_id(port, protocol, toport, toaddr)
         if forward_id in _obj.settings["forward_ports"]:
-            raise FirewallError(ALREADY_ENABLED)
+            raise FirewallError(ALREADY_ENABLED,
+                                "'%s:%s:%s:%s' already in '%s'" % \
+                                (port, protocol, toport, toaddr, zone))
 
         mark = self._fw.new_mark()
         if _obj.applied:
@@ -1411,7 +1443,9 @@ class FirewallZone:
 
         forward_id = self.__forward_port_id(port, protocol, toport, toaddr)
         if not forward_id in _obj.settings["forward_ports"]:
-            raise FirewallError(NOT_ENABLED)
+            raise FirewallError(NOT_ENABLED,
+                                "'%s:%s:%s:%s' not in '%s'" % \
+                                (port, protocol, toport, toaddr, zone))
 
         mark = _obj.settings["forward_ports"][forward_id]["mark"]
 
@@ -1485,12 +1519,14 @@ class FirewallZone:
 
     def add_icmp_block(self, zone, icmp, timeout=0, sender=None):
         _zone = self._fw.check_zone(zone)
+        self._fw.check_timeout(timeout)
         self._fw.check_panic()
         _obj = self._zones[_zone]
 
         icmp_id = self.__icmp_block_id(icmp)
         if icmp_id in _obj.settings["icmp_blocks"]:
-            raise FirewallError(ALREADY_ENABLED)
+            raise FirewallError(ALREADY_ENABLED,
+                                "'%s' already in '%s'" % (icmp, zone))
 
         if _obj.applied:
             self.__icmp_block(True, _zone, icmp)
@@ -1507,7 +1543,8 @@ class FirewallZone:
 
         icmp_id = self.__icmp_block_id(icmp)
         if not icmp_id in _obj.settings["icmp_blocks"]:
-            raise FirewallError(NOT_ENABLED)
+            raise FirewallError(NOT_ENABLED,
+                                "'%s' not in '%s'" % (icmp, zone))
 
         if _obj.applied:
             self.__icmp_block(False, _zone, icmp)
