@@ -68,6 +68,10 @@ INVALID_FAMILY      =  124
 INVALID_LOG_LEVEL   =  125
 INVALID_AUDIT_TYPE  =  126
 INVALID_MARK        =  127
+INVALID_CONTEXT     =  128
+INVALID_COMMAND     =  129
+INVALID_USER        =  130
+INVALID_UID         =  131
 
 MISSING_TABLE       =  200
 MISSING_CHAIN       =  201
@@ -85,16 +89,13 @@ UNKNOWN_ERROR       =  254
 import sys
 
 class FirewallError(Exception):
-    mod = sys.modules[__module__]
-    errors = dict([(getattr(mod,varname),varname)
-                   for varname in dir(mod)
-                   if not varname.startswith("_")])
-    codes = dict([(errors[code],code) for code in errors])
-
     def __init__(self, code, msg=None):
         self.code = code
         self.msg = msg
 
+    def __repr__(self):
+        return '%s(%r, %r)' % (self.__class__, self.code, self.msg)
+
     def __str__(self):
         if self.msg:
             return "%s: %s" % (self.errors[self.code], self.msg)
@@ -115,3 +116,11 @@ class FirewallError(Exception):
         return code
 
     get_code = staticmethod(get_code)
+
+mod = sys.modules[FirewallError.__module__]
+FirewallError.errors = { getattr(mod,varname) : varname
+                         for varname in dir(mod)
+                         if not varname.startswith("_") and \
+                         type(getattr(mod,varname)) == int }
+FirewallError.codes =  { FirewallError.errors[code] : code
+                         for code in FirewallError.errors }
