@@ -74,6 +74,7 @@ class FirewallDConfig(slip.dbus.service.Object):
         self.watcher.add_watch_dir(ETC_FIREWALLD_ZONES)
         self.watcher.add_watch_file(LOCKDOWN_WHITELIST)
         self.watcher.add_watch_file(FIREWALLD_DIRECT)
+        self.watcher.add_watch_file(FIREWALLD_CONF)
 
     @handle_exceptions
     def _init_vars(self):
@@ -113,11 +114,23 @@ class FirewallDConfig(slip.dbus.service.Object):
 
     @handle_exceptions
     def watch_updater(self, name):
-        if not name.endswith(".xml"):
-            raise FirewallError(INVALID_FILENAME, name)
+        if name == FIREWALLD_CONF:
+            old_props = self.GetAll(DBUS_INTERFACE_CONFIG)
+            log.debug1("config: Reloading firewalld config file '%s'",
+                       FIREWALLD_CONF)
+            self.config.update_firewalld_conf()
+            props = self.GetAll(DBUS_INTERFACE_CONFIG)
+            for key in props.keys():
+                if key in old_props and \
+                   old_props[key] == props[key]:
+                    del props[key]
+            if len(props) > 0:
+                self.PropertiesChanged(DBUS_INTERFACE_CONFIG, props, [])
+            return
 
-        if name.startswith(FIREWALLD_ICMPTYPES) or \
-                name.startswith(ETC_FIREWALLD_ICMPTYPES):
+        if (name.startswith(FIREWALLD_ICMPTYPES) or \
+            name.startswith(ETC_FIREWALLD_ICMPTYPES)) and \
+           name.endswith(".xml"):
             (what, obj) = self.config.update_icmptype_from_path(name)
             if what == "new":
                 self._addIcmpType(obj)
@@ -126,8 +139,9 @@ class FirewallDConfig(slip.dbus.service.Object):
             elif what == "update":
                 self._updateIcmpType(obj)
 
-        elif name.startswith(FIREWALLD_SERVICES) or \
-                name.startswith(ETC_FIREWALLD_SERVICES):
+        elif (name.startswith(FIREWALLD_SERVICES) or \
+              name.startswith(ETC_FIREWALLD_SERVICES)) and \
+             name.endswith(".xml"):
             (what, obj) = self.config.update_service_from_path(name)
             if what == "new":
                 self._addService(obj)
@@ -136,8 +150,9 @@ class FirewallDConfig(slip.dbus.service.Object):
             elif what == "update":
                 self._updateService(obj)
 
-        elif name.startswith(FIREWALLD_ZONES) or \
-                name.startswith(ETC_FIREWALLD_ZONES):
+        elif (name.startswith(FIREWALLD_ZONES) or \
+              name.startswith(ETC_FIREWALLD_ZONES)) and \
+             name.endswith(".xml"):
             (what, obj) = self.config.update_zone_from_path(name)
             if what == "new":
                 self._addZone(obj)
@@ -313,8 +328,8 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def Get(self, interface_name, property_name, sender=None):
         # get a property
-        interface_name = dbus_to_python(interface_name)
-        property_name = dbus_to_python(property_name)
+        interface_name = dbus_to_python(interface_name, str)
+        property_name = dbus_to_python(property_name, str)
         log.debug1("config.Get('%s', '%s')", interface_name, property_name)
 
         if interface_name != DBUS_INTERFACE_CONFIG:
@@ -328,7 +343,7 @@ class FirewallDConfig(slip.dbus.service.Object):
                          out_signature='a{sv}')
     @dbus_handle_exceptions
     def GetAll(self, interface_name, sender=None):
-        interface_name = dbus_to_python(interface_name)
+        interface_name = dbus_to_python(interface_name, str)
         log.debug1("config.GetAll('%s')", interface_name)
 
         if interface_name != DBUS_INTERFACE_CONFIG:
@@ -347,8 +362,8 @@ class FirewallDConfig(slip.dbus.service.Object):
     @dbus_service_method(dbus.PROPERTIES_IFACE, in_signature='ssv')
     @dbus_handle_exceptions
     def Set(self, interface_name, property_name, new_value, sender=None):
-        interface_name = dbus_to_python(interface_name)
-        property_name = dbus_to_python(property_name)
+        interface_name = dbus_to_python(interface_name, str)
+        property_name = dbus_to_python(property_name, str)
         new_value = dbus_to_python(new_value)
         log.debug1("config.Set('%s', '%s', '%s')", interface_name,
                    property_name, new_value)
@@ -359,7 +374,7 @@ class FirewallDConfig(slip.dbus.service.Object):
                 "org.freedesktop.DBus.Error.UnknownInterface: "
                 "FirewallD does not implement %s" % interface_name)
 
-        if property_name in [ "MinimalMark", "CleanupOnExit" ]:
+        if property_name in [ "MinimalMark", "CleanupOnExit", "Lockdown" ]:
             if property_name == "MinimalMark":
                 try:
                     foo = int(new_value)
@@ -370,7 +385,7 @@ class FirewallDConfig(slip.dbus.service.Object):
             except:
                 raise FirewallError(INVALID_VALUE, "'%s' for %s" % \
                                             (new_value, property_name))
-            if property_name in [ "CleanupOnExit" ]:
+            if property_name in [ "CleanupOnExit", "Lockdown" ]:
                 if new_value.lower() not in [ "yes", "no", "true", "false" ]:
                     raise FirewallError(INVALID_VALUE, "'%s' for %s" % \
                                             (new_value, property_name))
@@ -378,7 +393,7 @@ class FirewallDConfig(slip.dbus.service.Object):
             self.config.get_firewalld_conf().write()
             self.PropertiesChanged(interface_name,
                                    { property_name: new_value }, [ ])
-        elif property_name in [ "DefaultZone", "Lockdown" ]:
+        elif property_name in [ "DefaultZone" ]:
             raise dbus.exceptions.DBusException(
                 "org.freedesktop.DBus.Error.PropertyReadOnly: "
                 "Property '%s' is read-only" % property_name)
@@ -437,7 +452,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     def getIcmpTypeByName(self, icmptype, sender=None):
         """object path of icmptype with given name
         """
-        icmptype = dbus_to_python(icmptype)
+        icmptype = dbus_to_python(icmptype, str)
         log.debug1("config.getIcmpTypeByName('%s')", icmptype)
         for obj in self.icmptypes:
             if obj.obj.name == icmptype:
@@ -451,7 +466,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     def addIcmpType(self, icmptype, settings, sender=None):
         """add icmptype with given name and settings
         """
-        icmptype = dbus_to_python(icmptype)
+        icmptype = dbus_to_python(icmptype, str)
         settings = dbus_to_python(settings)
         log.debug1("config.addIcmpType('%s')", icmptype)
         self.accessCheck(sender)
@@ -480,7 +495,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     def getServiceByName(self, service, sender=None):
         """object path of service with given name
         """
-        service = dbus_to_python(service)
+        service = dbus_to_python(service, str)
         log.debug1("config.getServiceByName('%s')", service)
         for obj in self.services:
             if obj.obj.name == service:
@@ -494,7 +509,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     def addService(self, service, settings, sender=None):
         """add service with given name and settings
         """
-        service = dbus_to_python(service)
+        service = dbus_to_python(service, str)
         settings = dbus_to_python(settings)
         log.debug1("config.addService('%s')", service)
         self.accessCheck(sender)
@@ -523,7 +538,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     def getZoneByName(self, zone, sender=None):
         """object path of zone with given name
         """
-        zone = dbus_to_python(zone)
+        zone = dbus_to_python(zone, str)
         log.debug1("config.getZoneByName('%s')", zone)
         for obj in self.zones:
             if obj.obj.name == zone:
@@ -536,7 +551,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     def getZoneOfInterface(self, iface, sender=None):
         """name of zone the given interface belongs to
         """
-        iface = dbus_to_python(iface)
+        iface = dbus_to_python(iface, str)
         log.debug1("config.getZoneOfInterface('%s')", iface)
         ret = []
         for obj in self.zones:
@@ -554,7 +569,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     def getZoneOfSource(self, source, sender=None):
         """name of zone the given source belongs to
         """
-        source = dbus_to_python(source)
+        source = dbus_to_python(source, str)
         log.debug1("config.getZoneOfSource('%s')", source)
         ret = []
         for obj in self.zones:
@@ -573,7 +588,7 @@ class FirewallDConfig(slip.dbus.service.Object):
     def addZone(self, zone, settings, sender=None):
         """add zone with given name and settings
         """
-        zone = dbus_to_python(zone)
+        zone = dbus_to_python(zone, str)
         settings = dbus_to_python(settings)
         log.debug1("config.addZone('%s')", zone)
         self.accessCheck(sender)
