@@ -77,8 +77,8 @@ class FirewallDConfigService(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def Get(self, interface_name, property_name, sender=None):
         # get a property
-        interface_name = dbus_to_python(interface_name)
-        property_name = dbus_to_python(property_name)
+        interface_name = dbus_to_python(interface_name, str)
+        property_name = dbus_to_python(property_name, str)
         log.debug1("config.service.%d.Get('%s', '%s')", self.id,
                    interface_name, property_name)
 
@@ -107,7 +107,7 @@ class FirewallDConfigService(slip.dbus.service.Object):
                          out_signature='a{sv}')
     @dbus_handle_exceptions
     def GetAll(self, interface_name, sender=None):
-        interface_name = dbus_to_python(interface_name)
+        interface_name = dbus_to_python(interface_name, str)
         log.debug1("config.service.%d.GetAll('%s')", self.id, interface_name)
 
         if interface_name != DBUS_INTERFACE_CONFIG_SERVICE:
@@ -126,8 +126,8 @@ class FirewallDConfigService(slip.dbus.service.Object):
     @dbus_service_method(dbus.PROPERTIES_IFACE, in_signature='ssv')
     @dbus_handle_exceptions
     def Set(self, interface_name, property_name, new_value, sender=None):
-        interface_name = dbus_to_python(interface_name)
-        property_name = dbus_to_python(property_name)
+        interface_name = dbus_to_python(interface_name, str)
+        property_name = dbus_to_python(property_name, str)
         new_value = dbus_to_python(new_value)
         log.debug1("config.service.%d.Set('%s', '%s', '%s')", self.id,
                    interface_name, property_name, new_value)
@@ -207,7 +207,7 @@ class FirewallDConfigService(slip.dbus.service.Object):
     def rename(self, name, sender=None):
         """rename service
         """
-        name = dbus_to_python(name)
+        name = dbus_to_python(name, str)
         log.debug1("config.service.%d.rename('%s')", self.id, name)
         self.parent.accessCheck(sender)
         new_service = self.config.rename_service(self.obj, name)
