@@ -164,8 +164,8 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def Get(self, interface_name, property_name, sender=None):
         # get a property
-        interface_name = dbus_to_python(interface_name)
-        property_name = dbus_to_python(property_name)
+        interface_name = dbus_to_python(interface_name, str)
+        property_name = dbus_to_python(property_name, str)
         log.debug1("Get('%s', '%s')", interface_name, property_name)
 
         if interface_name != DBUS_INTERFACE:
@@ -179,7 +179,7 @@ class FirewallD(slip.dbus.service.Object):
                          out_signature='a{sv}')
     @dbus_handle_exceptions
     def GetAll(self, interface_name, sender=None):
-        interface_name = dbus_to_python(interface_name)
+        interface_name = dbus_to_python(interface_name, str)
         log.debug1("GetAll('%s')", interface_name)
 
         if interface_name != DBUS_INTERFACE:
@@ -202,8 +202,8 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_service_method(dbus.PROPERTIES_IFACE, in_signature='ssv')
     @dbus_handle_exceptions
     def Set(self, interface_name, property_name, new_value, sender=None):
-        interface_name = dbus_to_python(interface_name)
-        property_name = dbus_to_python(property_name)
+        interface_name = dbus_to_python(interface_name, str)
+        property_name = dbus_to_python(property_name, str)
         new_value = dbus_to_python(new_value)
         log.debug1("Set('%s', '%s', '%s')", interface_name, property_name,
                    new_value)
@@ -321,7 +321,7 @@ class FirewallD(slip.dbus.service.Object):
     def addLockdownWhitelistCommand(self, command, sender=None):
         """Add lockdown command
         """
-        command = dbus_to_python(command)
+        command = dbus_to_python(command, str)
         log.debug1("policies.addLockdownWhitelistCommand('%s')" % command)
         self.accessCheck(sender)
         self.fw.policies.lockdown_whitelist.add_command(command)
@@ -334,7 +334,7 @@ class FirewallD(slip.dbus.service.Object):
     def removeLockdownWhitelistCommand(self, command, sender=None):
         """Remove lockdown command
         """
-        command = dbus_to_python(command)
+        command = dbus_to_python(command, str)
         log.debug1("policies.removeLockdownWhitelistCommand('%s')" % command)
         self.accessCheck(sender)
         self.fw.policies.lockdown_whitelist.remove_command(command)
@@ -347,7 +347,7 @@ class FirewallD(slip.dbus.service.Object):
     def queryLockdownWhitelistCommand(self, command, sender=None):
         """Query lockdown command
         """
-        command = dbus_to_python(command)
+        command = dbus_to_python(command, str)
         log.debug1("policies.queryLockdownWhitelistCommand('%s')" % command)
         # no access check here
         return self.fw.policies.lockdown_whitelist.has_command(command)
@@ -382,7 +382,7 @@ class FirewallD(slip.dbus.service.Object):
     def addLockdownWhitelistUid(self, uid, sender=None):
         """Add lockdown uid
         """
-        uid = int(uid)
+        uid = dbus_to_python(uid, int)
         log.debug1("policies.addLockdownWhitelistUid('%s')" % uid)
         self.accessCheck(sender)
         self.fw.policies.lockdown_whitelist.add_uid(uid)
@@ -395,7 +395,7 @@ class FirewallD(slip.dbus.service.Object):
     def removeLockdownWhitelistUid(self, uid, sender=None):
         """Remove lockdown uid
         """
-        uid = int(uid)
+        uid = dbus_to_python(uid, int)
         log.debug1("policies.removeLockdownWhitelistUid('%s')" % uid)
         self.accessCheck(sender)
         self.fw.policies.lockdown_whitelist.remove_uid(uid)
@@ -408,7 +408,7 @@ class FirewallD(slip.dbus.service.Object):
     def queryLockdownWhitelistUid(self, uid, sender=None):
         """Query lockdown uid
         """
-        uid = int(uid)
+        uid = dbus_to_python(uid, int)
         log.debug1("policies.queryLockdownWhitelistUid('%s')" % uid)
         # no access check here
         return self.fw.policies.lockdown_whitelist.has_uid(uid)
@@ -443,7 +443,7 @@ class FirewallD(slip.dbus.service.Object):
     def addLockdownWhitelistUser(self, user, sender=None):
         """Add lockdown user
         """
-        user = dbus_to_python(user)
+        user = dbus_to_python(user, str)
         log.debug1("policies.addLockdownWhitelistUser('%s')" % user)
         self.accessCheck(sender)
         self.fw.policies.lockdown_whitelist.add_user(user)
@@ -456,7 +456,7 @@ class FirewallD(slip.dbus.service.Object):
     def removeLockdownWhitelistUser(self, user, sender=None):
         """Remove lockdown user
         """
-        user = dbus_to_python(user)
+        user = dbus_to_python(user, str)
         log.debug1("policies.removeLockdownWhitelistUser('%s')" % user)
         self.accessCheck(sender)
         self.fw.policies.lockdown_whitelist.remove_user(user)
@@ -469,7 +469,7 @@ class FirewallD(slip.dbus.service.Object):
     def queryLockdownWhitelistUser(self, user, sender=None):
         """Query lockdown user
         """
-        user = dbus_to_python(user)
+        user = dbus_to_python(user, str)
         log.debug1("policies.queryLockdownWhitelistUser('%s')" % user)
         # no access check here
         return self.fw.policies.lockdown_whitelist.has_user(user)
@@ -504,7 +504,7 @@ class FirewallD(slip.dbus.service.Object):
     def addLockdownWhitelistContext(self, context, sender=None):
         """Add lockdown context
         """
-        context = dbus_to_python(context)
+        context = dbus_to_python(context, str)
         log.debug1("policies.addLockdownWhitelistContext('%s')" % context)
         self.accessCheck(sender)
         self.fw.policies.lockdown_whitelist.add_context(context)
@@ -517,7 +517,7 @@ class FirewallD(slip.dbus.service.Object):
     def removeLockdownWhitelistContext(self, context, sender=None):
         """Remove lockdown context
         """
-        context = dbus_to_python(context)
+        context = dbus_to_python(context, str)
         log.debug1("policies.removeLockdownWhitelistContext('%s')" % context)
         self.accessCheck(sender)
         self.fw.policies.lockdown_whitelist.remove_context(context)
@@ -530,7 +530,7 @@ class FirewallD(slip.dbus.service.Object):
     def queryLockdownWhitelistContext(self, context, sender=None):
         """Query lockdown context
         """
-        context = dbus_to_python(context)
+        context = dbus_to_python(context, str)
         log.debug1("policies.queryLockdownWhitelistContext('%s')" % context)
         # no access check here
         return self.fw.policies.lockdown_whitelist.has_context(context)
@@ -615,7 +615,7 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def getZoneSettings(self, zone, sender=None):
         # returns zone settings for zone
-        zone = dbus_to_python(zone)
+        zone = dbus_to_python(zone, str)
         log.debug1("getZoneSettings(%s)", zone)
         return self.fw.zone.get_zone(zone).export_config()
 
@@ -636,7 +636,7 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def getServiceSettings(self, service, sender=None):
         # returns service settings for service
-        service = dbus_to_python(service)
+        service = dbus_to_python(service, str)
         log.debug1("getServiceSettings(%s)", service)
         return self.fw.service.get_service(service).export_config()
 
@@ -657,7 +657,7 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def getIcmpTypeSettings(self, icmptype, sender=None):
         # returns icmptype settings for icmptype
-        icmptype = dbus_to_python(icmptype)
+        icmptype = dbus_to_python(icmptype, str)
         log.debug1("getIcmpTypeSettings(%s)", icmptype)
         return self.fw.icmptype.get_icmptype(icmptype).export_config()
 
@@ -678,7 +678,7 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def setDefaultZone(self, zone, sender=None):
         # set the system default zone
-        zone = dbus_to_python(zone)
+        zone = dbus_to_python(zone, str)
         log.debug1("setDefaultZone('%s')" % zone)
         self.accessCheck(sender)
         self.fw.set_default_zone(zone)
@@ -735,7 +735,7 @@ class FirewallD(slip.dbus.service.Object):
                 Name of the interface
         :Returns: str. The name of the zone.
         """
-        interface = dbus_to_python(interface)
+        interface = dbus_to_python(interface, str)
         log.debug1("zone.getZoneOfInterface('%s')" % interface)
         zone = self.fw.zone.get_zone_of_interface(interface)
         if zone:
@@ -748,7 +748,7 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def getZoneOfSource(self, source, sender=None):
         #Return the zone an source belongs to.
-        source = dbus_to_python(source)
+        source = dbus_to_python(source, str)
         log.debug1("zone.getZoneOfSource('%s')" % source)
         zone = self.fw.zone.get_zone_of_source(source)
         if zone:
@@ -775,8 +775,8 @@ class FirewallD(slip.dbus.service.Object):
         """Add an interface to a zone.
         If zone is empty, use default zone.
         """
-        zone = dbus_to_python(zone)
-        interface = dbus_to_python(interface)
+        zone = dbus_to_python(zone, str)
+        interface = dbus_to_python(interface, str)
         log.debug1("zone.addInterface('%s', '%s')" % (zone, interface))
         self.accessCheck(sender)
         _zone = self.fw.zone.add_interface(zone, interface, sender)
@@ -794,8 +794,8 @@ class FirewallD(slip.dbus.service.Object):
 
         This function is deprecated, use changeZoneOfInterface instead
         """
-        zone = dbus_to_python(zone)
-        interface = dbus_to_python(interface)
+        zone = dbus_to_python(zone, str)
+        interface = dbus_to_python(interface, str)
         return self.changeZoneOfInterface(zone, interface, sender)
 
     @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG)
@@ -806,8 +806,8 @@ class FirewallD(slip.dbus.service.Object):
         """Change a zone an interface is part of.
         If zone is empty, use default zone.
         """
-        zone = dbus_to_python(zone)
-        interface = dbus_to_python(interface)
+        zone = dbus_to_python(zone, str)
+        interface = dbus_to_python(interface, str)
         log.debug1("zone.changeZoneOfInterface('%s', '%s')" % (zone, interface))
         self.accessCheck(sender)
         _zone = self.fw.zone.change_zone_of_interface(zone, interface, sender)
@@ -823,8 +823,8 @@ class FirewallD(slip.dbus.service.Object):
         """Remove interface from a zone.
         If zone is empty, remove from zone the interface belongs to.
         """
-        zone = dbus_to_python(zone)
-        interface = dbus_to_python(interface)
+        zone = dbus_to_python(zone, str)
+        interface = dbus_to_python(interface, str)
         log.debug1("zone.removeInterface('%s', '%s')" % (zone, interface))
         self.accessCheck(sender)
         _zone = self.fw.zone.remove_interface(zone, interface)
@@ -840,8 +840,8 @@ class FirewallD(slip.dbus.service.Object):
         """Return true if an interface is in a zone.
         If zone is empty, use default zone.
         """
-        zone = dbus_to_python(zone)
-        interface = dbus_to_python(interface)
+        zone = dbus_to_python(zone, str)
+        interface = dbus_to_python(interface, str)
         log.debug1("zone.queryInterface('%s', '%s')" % (zone, interface))
         return self.fw.zone.query_interface(zone, interface)
 
@@ -855,7 +855,7 @@ class FirewallD(slip.dbus.service.Object):
         """
         # TODO: should be renamed to listInterfaces()
         # because is called by firewall-cmd --zone --list-interfaces
-        zone = dbus_to_python(zone)
+        zone = dbus_to_python(zone, str)
         log.debug1("zone.getInterfaces('%s')" % (zone))
         return self.fw.zone.list_interfaces(zone)
 
@@ -896,8 +896,8 @@ class FirewallD(slip.dbus.service.Object):
         """Add a source to a zone.
         If zone is empty, use default zone.
         """
-        zone = dbus_to_python(zone)
-        source = dbus_to_python(source)
+        zone = dbus_to_python(zone, str)
+        source = dbus_to_python(source, str)
         log.debug1("zone.addSource('%s', '%s')" % (zone, source))
         self.accessCheck(sender)
         _zone = self.fw.zone.add_source(zone, source, sender)
@@ -913,8 +913,8 @@ class FirewallD(slip.dbus.service.Object):
         """Change a zone an source is part of.
         If zone is empty, use default zone.
         """
-        zone = dbus_to_python(zone)
-        source = dbus_to_python(source)
+        zone = dbus_to_python(zone, str)
+        source = dbus_to_python(source, str)
         log.debug1("zone.changeZoneOfSource('%s', '%s')" % (zone, source))
         self.accessCheck(sender)
         _zone = self.fw.zone.change_zone_of_source(zone, source, sender)
@@ -930,8 +930,8 @@ class FirewallD(slip.dbus.service.Object):
         """Remove source from a zone.
         If zone is empty, remove from zone the source belongs to.
         """
-        zone = dbus_to_python(zone)
-        source = dbus_to_python(source)
+        zone = dbus_to_python(zone, str)
+        source = dbus_to_python(source, str)
         log.debug1("zone.removeSource('%s', '%s')" % (zone, source))
         self.accessCheck(sender)
         _zone = self.fw.zone.remove_source(zone, source)
@@ -947,8 +947,8 @@ class FirewallD(slip.dbus.service.Object):
         """Return true if an source is in a zone.
         If zone is empty, use default zone.
         """
-        zone = dbus_to_python(zone)
-        source = dbus_to_python(source)
+        zone = dbus_to_python(zone, str)
+        source = dbus_to_python(source, str)
         log.debug1("zone.querySource('%s', '%s')" % (zone, source))
         return self.fw.zone.query_source(zone, source)
 
@@ -962,7 +962,7 @@ class FirewallD(slip.dbus.service.Object):
         """
         # TODO: should be renamed to listSources()
         # because is called by firewall-cmd --zone --list-sources
-        zone = dbus_to_python(zone)
+        zone = dbus_to_python(zone, str)
         log.debug1("zone.getSources('%s')" % (zone))
         return self.fw.zone.list_sources(zone)
 
@@ -998,13 +998,12 @@ class FirewallD(slip.dbus.service.Object):
                          out_signature='s')
     @dbus_handle_exceptions
     def addRichRule(self, zone, rule, timeout, sender=None):
-        # timeout not possible for masquerade
-        zone = dbus_to_python(zone)
-        rule = dbus_to_python(rule)
-        timeout = dbus_to_python(timeout)
+        zone = dbus_to_python(zone, str)
+        rule = dbus_to_python(rule, str)
+        timeout = dbus_to_python(timeout, int)
         log.debug1("zone.addRichRule('%s', '%s')" % (zone, rule))
         obj = Rich_Rule(rule_str=rule)
-        _zone = self.fw.zone.add_rule(zone, obj)
+        _zone = self.fw.zone.add_rule(zone, obj, timeout)
 
         if timeout > 0:
             tag = GLib.timeout_add_seconds(timeout, self.disableTimedRichRule,
@@ -1019,9 +1018,8 @@ class FirewallD(slip.dbus.service.Object):
                          out_signature='s')
     @dbus_handle_exceptions
     def removeRichRule(self, zone, rule, sender=None):
-        # timeout not possible for masquerade
-        zone = dbus_to_python(zone)
-        rule = dbus_to_python(rule)
+        zone = dbus_to_python(zone, str)
+        rule = dbus_to_python(rule, str)
         log.debug1("zone.removeRichRule('%s', '%s')" % (zone, rule))
         obj = Rich_Rule(rule_str=rule)
         _zone = self.fw.zone.remove_rule(zone, obj)
@@ -1034,9 +1032,8 @@ class FirewallD(slip.dbus.service.Object):
                          out_signature='b')
     @dbus_handle_exceptions
     def queryRichRule(self, zone, rule, sender=None):
-        # timeout not possible for masquerade
-        zone = dbus_to_python(zone)
-        rule = dbus_to_python(rule)
+        zone = dbus_to_python(zone, str)
+        rule = dbus_to_python(rule, str)
         log.debug1("zone.queryRichRule('%s', '%s')" % (zone, rule))
         obj = Rich_Rule(rule_str=rule)
         return self.fw.zone.query_rule(zone, obj)
@@ -1049,7 +1046,7 @@ class FirewallD(slip.dbus.service.Object):
         # returns the list of enabled rich rules for zone
         # TODO: should be renamed to listRichRules()
         # because is called by firewall-cmd --zone --list-rich-rules
-        zone = dbus_to_python(zone)
+        zone = dbus_to_python(zone, str)
         log.debug1("zone.getRichRules('%s')" % (zone))
         return self.fw.zone.list_rules(zone)
 
@@ -1080,9 +1077,9 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def addService(self, zone, service, timeout, sender=None):
         # enables service <service> if not enabled already for zone
-        zone = dbus_to_python(zone)
-        service = dbus_to_python(service)
-        timeout = dbus_to_python(timeout)
+        zone = dbus_to_python(zone, str)
+        service = dbus_to_python(service, str)
+        timeout = dbus_to_python(timeout, int)
         log.debug1("zone.addService('%s', '%s', %d)" % (zone, service, timeout))
         self.accessCheck(sender)
 
@@ -1102,8 +1099,8 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def removeService(self, zone, service, sender=None):
         # disables service for zone
-        zone = dbus_to_python(zone)
-        service = dbus_to_python(service)
+        zone = dbus_to_python(zone, str)
+        service = dbus_to_python(service, str)
         log.debug1("zone.removeService('%s', '%s')" % (zone, service))
         self.accessCheck(sender)
 
@@ -1119,8 +1116,8 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def queryService(self, zone, service, sender=None):
         # returns true if a service is enabled for zone
-        zone = dbus_to_python(zone)
-        service = dbus_to_python(service)
+        zone = dbus_to_python(zone, str)
+        service = dbus_to_python(service, str)
         log.debug1("zone.queryService('%s', '%s')" % (zone, service))
         return self.fw.zone.query_service(zone, service)
 
@@ -1132,7 +1129,7 @@ class FirewallD(slip.dbus.service.Object):
         # returns the list of enabled services for zone
         # TODO: should be renamed to listServices()
         # because is called by firewall-cmd --zone --list-services
-        zone = dbus_to_python(zone)
+        zone = dbus_to_python(zone, str)
         log.debug1("zone.getServices('%s')" % (zone))
         return self.fw.zone.list_services(zone)
 
@@ -1165,10 +1162,10 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def addPort(self, zone, port, protocol, timeout, sender=None):
         # adds port <port> <protocol> if not enabled already to zone
-        zone = dbus_to_python(zone)
-        port = dbus_to_python(port)
-        protocol = dbus_to_python(protocol)
-        timeout = dbus_to_python(timeout)
+        zone = dbus_to_python(zone, str)
+        port = dbus_to_python(port, str)
+        protocol = dbus_to_python(protocol, str)
+        timeout = dbus_to_python(timeout, int)
         log.debug1("zone.enablePort('%s', '%s', '%s')" % \
                        (zone, port, protocol))
         self.accessCheck(sender)
@@ -1188,9 +1185,9 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def removePort(self, zone, port, protocol, sender=None):
         # removes port<port> <protocol> if enabled from zone
-        zone = dbus_to_python(zone)
-        port = dbus_to_python(port)
-        protocol = dbus_to_python(protocol)
+        zone = dbus_to_python(zone, str)
+        port = dbus_to_python(port, str)
+        protocol = dbus_to_python(protocol, str)
         log.debug1("zone.removePort('%s', '%s', '%s')" % \
                        (zone, port, protocol))
         self.accessCheck(sender)
@@ -1206,9 +1203,9 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def queryPort(self, zone, port, protocol, sender=None):
         # returns true if a port is enabled for zone
-        zone = dbus_to_python(zone)
-        port = dbus_to_python(port)
-        protocol = dbus_to_python(protocol)
+        zone = dbus_to_python(zone, str)
+        port = dbus_to_python(port, str)
+        protocol = dbus_to_python(protocol, str)
         log.debug1("zone.queryPort('%s', '%s', '%s')" % (zone, port, protocol))
         return self.fw.zone.query_port(zone, port, protocol)
 
@@ -1220,7 +1217,7 @@ class FirewallD(slip.dbus.service.Object):
         # returns the list of enabled ports
         # TODO: should be renamed to listPorts()
         # because is called by firewall-cmd --zone --list-ports
-        zone = dbus_to_python(zone)
+        zone = dbus_to_python(zone, str)
         log.debug1("zone.getPorts('%s')" % (zone))
         return self.fw.zone.list_ports(zone)
 
@@ -1252,8 +1249,8 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def addMasquerade(self, zone, timeout, sender=None):
         # adds masquerade if not added already
-        zone = dbus_to_python(zone)
-        timeout = int(timeout)
+        zone = dbus_to_python(zone, str)
+        timeout = dbus_to_python(timeout, int)
         log.debug1("zone.addMasquerade('%s')" % (zone))
         self.accessCheck(sender)
         _zone = self.fw.zone.add_masquerade(zone, timeout, sender)
@@ -1272,7 +1269,7 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def removeMasquerade(self, zone, sender=None):
         # removes masquerade
-        zone = dbus_to_python(zone)
+        zone = dbus_to_python(zone, str)
         log.debug1("zone.removeMasquerade('%s')" % (zone))
         self.accessCheck(sender)
         _zone = self.fw.zone.remove_masquerade(zone)
@@ -1287,7 +1284,7 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def queryMasquerade(self, zone, sender=None):
         # returns true if a masquerade is added
-        zone = dbus_to_python(zone)
+        zone = dbus_to_python(zone, str)
         log.debug1("zone.queryMasquerade('%s')" % (zone))
         return self.fw.zone.query_masquerade(zone)
 
@@ -1318,12 +1315,12 @@ class FirewallD(slip.dbus.service.Object):
     def addForwardPort(self, zone, port, protocol, toport, toaddr, timeout,
                        sender=None):
         # add forward port if not enabled already for zone
-        zone = dbus_to_python(zone)
-        port = dbus_to_python(port)
-        protocol = dbus_to_python(protocol)
-        toport = dbus_to_python(toport)
-        toaddr = dbus_to_python(toaddr)
-        timeout = dbus_to_python(timeout)
+        zone = dbus_to_python(zone, str)
+        port = dbus_to_python(port, str)
+        protocol = dbus_to_python(protocol, str)
+        toport = dbus_to_python(toport, str)
+        toaddr = dbus_to_python(toaddr, str)
+        timeout = dbus_to_python(timeout, int)
         log.debug1("zone.addForwardPort('%s', '%s', '%s', '%s', '%s')" % \
                        (zone, port, protocol, toport, toaddr))
         self.accessCheck(sender)
@@ -1347,11 +1344,11 @@ class FirewallD(slip.dbus.service.Object):
     def removeForwardPort(self, zone, port, protocol, toport, toaddr,
                           sender=None):
         # remove forward port from zone
-        zone = dbus_to_python(zone)
-        port = dbus_to_python(port)
-        protocol = dbus_to_python(protocol)
-        toport = dbus_to_python(toport)
-        toaddr = dbus_to_python(toaddr)
+        zone = dbus_to_python(zone, str)
+        port = dbus_to_python(port, str)
+        protocol = dbus_to_python(protocol, str)
+        toport = dbus_to_python(toport, str)
+        toaddr = dbus_to_python(toaddr, str)
         log.debug1("zone.removeForwardPort('%s', '%s', '%s', '%s', '%s')" % \
                        (zone, port, protocol, toport, toaddr))
         self.accessCheck(sender)
@@ -1369,11 +1366,11 @@ class FirewallD(slip.dbus.service.Object):
     def queryForwardPort(self, zone, port, protocol, toport, toaddr,
                          sender=None):
         # returns true if a forward port is enabled for zone
-        zone = dbus_to_python(zone)
-        port = dbus_to_python(port)
-        protocol = dbus_to_python(protocol)
-        toport = dbus_to_python(toport)
-        toaddr = dbus_to_python(toaddr)
+        zone = dbus_to_python(zone, str)
+        port = dbus_to_python(port, str)
+        protocol = dbus_to_python(protocol, str)
+        toport = dbus_to_python(toport, str)
+        toaddr = dbus_to_python(toaddr, str)
         log.debug1("zone.queryForwardPort('%s', '%s', '%s', '%s', '%s')" % \
                        (zone, port, protocol, toport, toaddr))
         return self.fw.zone.query_forward_port(zone, port, protocol, toport,
@@ -1387,7 +1384,7 @@ class FirewallD(slip.dbus.service.Object):
         # returns the list of enabled ports for zone
         # TODO: should be renamed to listForwardPorts()
         # because is called by firewall-cmd --zone --list-forward-ports
-        zone = dbus_to_python(zone)
+        zone = dbus_to_python(zone, str)
         log.debug1("zone.getForwardPorts('%s')" % (zone))
         return self.fw.zone.list_forward_ports(zone)
 
@@ -1421,9 +1418,9 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def addIcmpBlock(self, zone, icmp, timeout, sender=None):
         # add icmpblock <icmp> if not enabled already for zone
-        zone = dbus_to_python(zone)
-        icmp = dbus_to_python(icmp)
-        timeout = dbus_to_python(timeout)
+        zone = dbus_to_python(zone, str)
+        icmp = dbus_to_python(icmp, str)
+        timeout = dbus_to_python(timeout, int)
         log.debug1("zone.enableIcmpBlock('%s', '%s')" % (zone, icmp))
         self.accessCheck(sender)
         _zone = self.fw.zone.add_icmp_block(zone, icmp, timeout, sender)
@@ -1442,8 +1439,8 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def removeIcmpBlock(self, zone, icmp, sender=None):
         # removes icmpBlock from zone
-        zone = dbus_to_python(zone)
-        icmp = dbus_to_python(icmp)
+        zone = dbus_to_python(zone, str)
+        icmp = dbus_to_python(icmp, str)
         log.debug1("zone.removeIcmpBlock('%s', '%s')" % (zone, icmp))
         self.accessCheck(sender)
         _zone = self.fw.zone.remove_icmp_block(zone, icmp)
@@ -1458,8 +1455,8 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def queryIcmpBlock(self, zone, icmp, sender=None):
         # returns true if a icmp is enabled for zone
-        zone = dbus_to_python(zone)
-        icmp = dbus_to_python(icmp)
+        zone = dbus_to_python(zone, str)
+        icmp = dbus_to_python(icmp, str)
         log.debug1("zone.queryIcmpBlock('%s', '%s')" % (zone, icmp))
         return self.fw.zone.query_icmp_block(zone, icmp)
 
@@ -1471,7 +1468,7 @@ class FirewallD(slip.dbus.service.Object):
         # returns the list of enabled icmpblocks
         # TODO: should be renamed to listIcmpBlocks()
         # because is called by firewall-cmd --zone --list-icmp-blocks
-        zone = dbus_to_python(zone)
+        zone = dbus_to_python(zone, str)
         log.debug1("zone.getIcmpBlocks('%s')" % (zone))
         return self.fw.zone.list_icmp_blocks(zone)
 
@@ -1498,9 +1495,9 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def addChain(self, ipv, table, chain, sender=None):
         # inserts direct chain
-        ipv = dbus_to_python(ipv)
-        table = dbus_to_python(table)
-        chain = dbus_to_python(chain)
+        ipv = dbus_to_python(ipv, str)
+        table = dbus_to_python(table, str)
+        chain = dbus_to_python(chain, str)
         log.debug1("direct.addChain('%s', '%s', '%s')" % (ipv, table, chain))
         self.accessCheck(sender)
         self.fw.direct.add_chain(ipv, table, chain)
@@ -1512,9 +1509,9 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def removeChain(self, ipv, table, chain, sender=None):
         # removes direct chain
-        ipv = dbus_to_python(ipv)
-        table = dbus_to_python(table)
-        chain = dbus_to_python(chain)
+        ipv = dbus_to_python(ipv, str)
+        table = dbus_to_python(table, str)
+        chain = dbus_to_python(chain, str)
         log.debug1("direct.removeChain('%s', '%s', '%s')" % (ipv, table, chain))
         self.accessCheck(sender)
         self.fw.direct.remove_chain(ipv, table, chain)
@@ -1526,9 +1523,9 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def queryChain(self, ipv, table, chain, sender=None):
         # returns true if a chain is enabled
-        ipv = dbus_to_python(ipv)
-        table = dbus_to_python(table)
-        chain = dbus_to_python(chain)
+        ipv = dbus_to_python(ipv, str)
+        table = dbus_to_python(table, str)
+        chain = dbus_to_python(chain, str)
         log.debug1("direct.queryChain('%s', '%s', '%s')" % (ipv, table, chain))
         return self.fw.direct.query_chain(ipv, table, chain)
 
@@ -1538,8 +1535,8 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def getChains(self, ipv, table, sender=None):
         # returns list of added chains
-        ipv = dbus_to_python(ipv)
-        table = dbus_to_python(table)
+        ipv = dbus_to_python(ipv, str)
+        table = dbus_to_python(table, str)
         log.debug1("direct.getChains('%s', '%s')" % (ipv, table))
         return self.fw.direct.get_chains(ipv, table)
 
@@ -1572,11 +1569,11 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def addRule(self, ipv, table, chain, priority, args, sender=None):
         # inserts direct rule
-        ipv = dbus_to_python(ipv)
-        table = dbus_to_python(table)
-        chain = dbus_to_python(chain)
-        priority = dbus_to_python(priority)
-        args = tuple( dbus_to_python(i) for i in args )
+        ipv = dbus_to_python(ipv, str)
+        table = dbus_to_python(table, str)
+        chain = dbus_to_python(chain, str)
+        priority = dbus_to_python(priority, int)
+        args = tuple( dbus_to_python(i, str) for i in args )
         log.debug1("direct.addRule('%s', '%s', '%s', %d, '%s')" % \
                        (ipv, table, chain, priority, "','".join(args)))
         self.accessCheck(sender)
@@ -1589,11 +1586,11 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def removeRule(self, ipv, table, chain, priority, args, sender=None):
         # removes direct rule
-        ipv = dbus_to_python(ipv)
-        table = dbus_to_python(table)
-        chain = dbus_to_python(chain)
-        priority = dbus_to_python(priority)
-        args = tuple( dbus_to_python(i) for i in args )
+        ipv = dbus_to_python(ipv, str)
+        table = dbus_to_python(table, str)
+        chain = dbus_to_python(chain, str)
+        priority = dbus_to_python(priority, int)
+        args = tuple( dbus_to_python(i, str) for i in args )
         log.debug1("direct.removeRule('%s', '%s', '%s', %d, '%s')" % \
                        (ipv, table, chain, priority, "','".join(args)))
         self.accessCheck(sender)
@@ -1606,9 +1603,9 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def removeRules(self, ipv, table, chain, sender=None):
         # removes direct rule
-        ipv = dbus_to_python(ipv)
-        table = dbus_to_python(table)
-        chain = dbus_to_python(chain)
+        ipv = dbus_to_python(ipv, str)
+        table = dbus_to_python(table, str)
+        chain = dbus_to_python(chain, str)
         log.debug1("direct.removeRules('%s', '%s', '%s')" % (ipv, table, chain))
         self.accessCheck(sender)
         for (priority, args) in self.fw.direct.get_rules(ipv, table, chain):
@@ -1621,11 +1618,11 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def queryRule(self, ipv, table, chain, priority, args, sender=None):
         # returns true if a rule is enabled
-        ipv = dbus_to_python(ipv)
-        table = dbus_to_python(table)
-        chain = dbus_to_python(chain)
-        priority = dbus_to_python(priority)
-        args = tuple( dbus_to_python(i) for i in args )
+        ipv = dbus_to_python(ipv, str)
+        table = dbus_to_python(table, str)
+        chain = dbus_to_python(chain, str)
+        priority = dbus_to_python(priority, int)
+        args = tuple( dbus_to_python(i, str) for i in args )
         log.debug1("directQueryRule('%s','%s', %d, '%s')" % \
                        (table, chain, priority, "','".join(args)))
         return self.fw.direct.query_rule(ipv, table, chain, priority, args)
@@ -1636,9 +1633,9 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def getRules(self, ipv, table, chain, sender=None):
         # returns list of added rules
-        ipv = dbus_to_python(ipv)
-        table = dbus_to_python(table)
-        chain = dbus_to_python(chain)
+        ipv = dbus_to_python(ipv, str)
+        table = dbus_to_python(table, str)
+        chain = dbus_to_python(chain, str)
         log.debug1("direct.getRules('%s', '%s', '%s')" % (ipv, table, chain))
         return self.fw.direct.get_rules(ipv, table, chain)
 
@@ -1673,8 +1670,8 @@ class FirewallD(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def passthrough(self, ipv, args, sender=None):
         # inserts direct rule
-        ipv = dbus_to_python(ipv)
-        args = tuple( dbus_to_python(i) for i in args )
+        ipv = dbus_to_python(ipv, str)
+        args = tuple( dbus_to_python(i, str) for i in args )
         log.debug1("direct.passthrough('%s', '%s')" % (ipv, "','".join(args)))
         self.accessCheck(sender)
         return self.fw.direct.passthrough(ipv, args)
